@@ -28,6 +28,14 @@ class Finder:
         :param reader: Reader instance
         :param retriever: Retriever instance
         """
+        logger.warning(
+            """DEPRECATION WARNINGS: 
+            1. The 'Finder' class will be deprecated in the next Haystack release in 
+            favour of a new `Pipeline` class that supports building custom search pipelines using Haystack components
+            including Retriever, Readers, and Generators.
+            For more details, please refer to the issue: https://github.com/deepset-ai/haystack/issues/544
+            2. The `question` parameter in search requests & results is renamed to `query`."""
+        )
         self.retriever = retriever
         self.reader = reader
         if self.reader is None and self.retriever is None:
@@ -46,6 +54,14 @@ class Finder:
         :return:
         """
 
+        logger.warning(
+            """DEPRECATION WARNINGS: 
+            1. The 'Finder' class will be deprecated in the next Haystack release in 
+            favour of a new `Pipeline` class that supports building custom search pipelines using Haystack components
+            including Retriever, Readers, and Generators.
+            For more details, please refer to the issue: https://github.com/deepset-ai/haystack/issues/544
+            2. The `question` parameter in search requests & results is renamed to `query`."""
+        )
         if self.retriever is None or self.reader is None:
             raise AttributeError("Finder.get_answers requires self.retriever AND self.reader")
 
@@ -63,9 +79,10 @@ class Finder:
         len_chars = sum([len(d.text) for d in documents])
         logger.info(f"Reader is looking for detailed answer in {len_chars} chars ...")
 
-        results = self.reader.predict(question=question,
+        results = self.reader.predict(query=question,
                                       documents=documents,
                                       top_k=top_k_reader)  # type: Dict[str, Any]
+        results["question"] = results["query"]
 
         # Add corresponding document_name and more meta data, if an answer contains the document_id
         for ans in results["answers"]:
@@ -131,7 +148,8 @@ class Finder:
         Returns a dict containing the following metrics:
             - ``"retriever_recall"``: Proportion of questions for which correct document is among retrieved documents
             - ``"retriever_map"``: Mean of average precision for each question. Rewards retrievers that give relevant
-              documents a higher rank. Considers all retrieved relevant documents.
+              documents a higher rank. Considers all retrieved relevant documents. Average precision is normalized by
+              the number of all relevant documents per query.
             - ``"retriever_mrr"``: Mean of reciprocal rank for each question. Rewards retrievers that give relevant
               documents a higher rank. Only considers the highest ranked relevant document.
             - ``"reader_top1_accuracy"``: Proportion of highest ranked predicted answers that overlap with corresponding correct answer
@@ -200,17 +218,21 @@ class Finder:
             # check if correct doc among retrieved docs
             found_relevant_doc = False
             relevant_docs_found = 0
+            current_avg_precision = 0.0
             for doc_idx, doc in enumerate(retrieved_docs):
                 if doc.id in question.multiple_document_ids:
                     relevant_docs_found += 1
                     if not found_relevant_doc:
                         counts["correct_retrievals"] += 1
                         counts["summed_reciprocal_rank_retriever"] += 1 / (doc_idx + 1)
-                    counts["summed_avg_precision_retriever"] += (1 / number_relevant_docs) \
-                                                                * (relevant_docs_found / (doc_idx + 1))
+                    current_avg_precision += relevant_docs_found / (doc_idx + 1)
+
                     found_relevant_doc = True
                     if relevant_docs_found == number_relevant_docs:
                         break
+            if found_relevant_doc:
+                all_relevant_docs = len(set(question.multiple_document_ids))
+                counts["summed_avg_precision_retriever"] += current_avg_precision / all_relevant_docs
 
             if found_relevant_doc:
                 questions_with_docs.append({
@@ -283,7 +305,8 @@ class Finder:
         Returns a dict containing the following metrics:
             - ``"retriever_recall"``: Proportion of questions for which correct document is among retrieved documents
             - ``"retriever_map"``: Mean of average precision for each question. Rewards retrievers that give relevant
-              documents a higher rank. Considers all retrieved relevant documents.
+              documents a higher rank. Considers all retrieved relevant documents. Average precision is normalized by
+              the number of all relevant documents per query.
             - ``"retriever_mrr"``: Mean of reciprocal rank for each question. Rewards retrievers that give relevant
               documents a higher rank. Only considers the highest ranked relevant document.
             - ``"reader_top1_accuracy"``: Proportion of highest ranked predicted answers that overlap with corresponding correct answer
@@ -356,7 +379,7 @@ class Finder:
         self.reader.return_no_answers = True
         reader_start_time = time.time()
         predictions = self.reader.predict_batch(questions_with_correct_doc,
-                                                top_k_per_question=top_k_reader, batch_size=batch_size)
+                                                top_k=top_k_reader, batch_size=batch_size)
         reader_total_time = time.time() - reader_start_time
 
         for pred in predictions:
@@ -472,4 +495,3 @@ class Finder:
             eval_results["reader_topk_no_answer_accuracy"] = None
 
         return eval_results
-
