@@ -44,7 +44,8 @@ class FARMReader(BaseReader):
         context_window_size: int = 150,
         batch_size: int = 50,
         use_gpu: bool = True,
-        no_ans_boost: Optional[float] = None,
+        no_ans_boost: float = 0.0,
+        return_no_answer: bool = False,
         top_k_per_candidate: int = 3,
         top_k_per_sample: int = 1,
         num_processes: Optional[int] = None,
@@ -63,9 +64,10 @@ class FARMReader(BaseReader):
                            to a value so only a single batch is used.
         :param use_gpu: Whether to use GPU (if available)
         :param no_ans_boost: How much the no_answer logit is boosted/increased.
-        If set to None (default), disables returning "no answer" predictions.
+        If set to 0 (default), the no_answer logit is not changed.
         If a negative number, there is a lower chance of "no_answer" being predicted.
         If a positive number, there is an increased chance of "no_answer"
+        :param return_no_answer: Whether to include no_answer predictions in the results.
         :param top_k_per_candidate: How many answers to extract for each candidate doc that is coming from the retriever (might be a long text).
         Note that this is not the number of "final answers" you will receive
         (see `top_k` in FARMReader.predict() or Finder.get_answers() for that)
@@ -85,11 +87,7 @@ class FARMReader(BaseReader):
 
         """
 
-        if no_ans_boost is None:
-            no_ans_boost = 0
-            self.return_no_answers = False
-        else:
-            self.return_no_answers = True
+        self.return_no_answers = return_no_answer
         self.top_k_per_candidate = top_k_per_candidate
         self.inferencer = QAInferencer.load(model_name_or_path, batch_size=batch_size, gpu=use_gpu,
                                           task_type="question_answering", max_seq_len=max_seq_len,
@@ -233,6 +231,29 @@ class FARMReader(BaseReader):
         self.inferencer.model = trainer.train()
         self.save(Path(save_dir))
 
+    def update_parameters(
+        self,
+        context_window_size: Optional[int] = None,
+        no_ans_boost: Optional[float] = None,
+        return_no_answer: Optional[bool] = None,
+        max_seq_len: Optional[int] = None,
+        doc_stride: Optional[int] = None,
+    ):
+        """
+        Hot update parameters of a loaded Reader. It may not to be safe when processing concurrent requests.
+        """
+        if no_ans_boost is not None:
+            self.inferencer.model.prediction_heads[0].no_ans_boost = no_ans_boost
+        if return_no_answer is not None:
+            self.return_no_answers = return_no_answer
+        if doc_stride is not None:
+            self.inferencer.processor.doc_stride = doc_stride
+        if context_window_size is not None:
+            self.inferencer.model.prediction_heads[0].context_window_size = context_window_size
+        if max_seq_len is not None:
+            self.inferencer.processor.max_seq_len = max_seq_len
+            self.max_seq_len = max_seq_len
+
     def save(self, directory: Path):
         """
         Saves the Reader model so that it can be reused at a later point in time.
@@ -243,16 +264,16 @@ class FARMReader(BaseReader):
         self.inferencer.model.save(directory)
         self.inferencer.processor.save(directory)
 
-    def predict_batch(self, question_doc_list: List[dict], top_k_per_question: int = None, batch_size: int = None):
+    def predict_batch(self, query_doc_list: List[dict], top_k: int = None, batch_size: int = None):
         """
-        Use loaded QA model to find answers for a list of questions in each question's supplied list of Document.
+        Use loaded QA model to find answers for a list of queries in each query's supplied list of Document.
 
         Returns list of dictionaries containing answers sorted by (desc.) probability
 
-        :param question_doc_list: List of dictionaries containing questions with their retrieved documents
-        :param top_k_per_question: The maximum number of answers to return for each question
+        :param query_doc_list: List of dictionaries containing queries with their retrieved documents
+        :param top_k: The maximum number of answers to return for each query
         :param batch_size: Number of samples the model receives in one batch for inference
-        :return: List of dictionaries containing question and answers
+        :return: List of dictionaries containing query and answers
         """
 
         # convert input to FARM format
@@ -261,20 +282,20 @@ class FARMReader(BaseReader):
         labels = []
 
         # build input objects for inference_from_objects
-        for question_with_docs in question_doc_list:
-            documents = question_with_docs["docs"]
-            question = question_with_docs["question"]
-            labels.append(question)
+        for query_with_docs in query_doc_list:
+            documents = query_with_docs["docs"]
+            query = query_with_docs["question"]
+            labels.append(query)
             number_of_docs.append(len(documents))
 
             for doc in documents:
                 cur = QAInput(doc_text=doc.text,
-                              questions=Question(text=question.question,
+                              questions=Question(text=query.question,
                                                  uid=doc.id))
                 inputs.append(cur)
 
         self.inferencer.batch_size = batch_size
-        # make predictions on all document-question pairs
+        # make predictions on all document-query pairs
         predictions = self.inferencer.inference_from_objects(
             objects=inputs, return_json=False, multiprocessing_chunksize=1
         )
@@ -290,11 +311,11 @@ class FARMReader(BaseReader):
 
         result = []
         for idx, group in enumerate(grouped_predictions):
-            answers, max_no_ans_gap = self._extract_answers_of_predictions(group, top_k_per_question)
-            question = group[0].question
+            answers, max_no_ans_gap = self._extract_answers_of_predictions(group, top_k)
+            query = group[0].question
             cur_label = labels[idx]
             result.append({
-                "question": question,
+                "query": query,
                 "no_ans_gap": max_no_ans_gap,
                 "answers": answers,
                 "label": cur_label
@@ -302,38 +323,39 @@ class FARMReader(BaseReader):
 
         return result
 
-    def predict(self, question: str, documents: List[Document], top_k: Optional[int] = None):
+    def predict(self, query: str, documents: List[Document], top_k: Optional[int] = None):
         """
-        Use loaded QA model to find answers for a question in the supplied list of Document.
+        Use loaded QA model to find answers for a query in the supplied list of Document.
 
         Returns dictionaries containing answers sorted by (desc.) probability.
         Example:
+         ```python
+            |{
+            |    'query': 'Who is the father of Arya Stark?',
+            |    'answers':[
+            |                 {'answer': 'Eddard,',
+            |                 'context': " She travels with her father, Eddard, to King's Landing when he is ",
+            |                 'offset_answer_start': 147,
+            |                 'offset_answer_end': 154,
+            |                 'probability': 0.9787139466668613,
+            |                 'score': None,
+            |                 'document_id': '1337'
+            |                 },...
+            |              ]
+            |}
+         ```
 
-            {'question': 'Who is the father of Arya Stark?',
-            'answers': [
-                         {'answer': 'Eddard,',
-                         'context': " She travels with her father, Eddard, to King's Landing when he is ",
-                         'offset_answer_start': 147,
-                         'offset_answer_end': 154,
-                         'probability': 0.9787139466668613,
-                         'score': None,
-                         'document_id': '1337'
-                         },
-                        ...
-                       ]
-            }
-
-        :param question: Question string
+        :param query: Query string
         :param documents: List of Document in which to search for the answer
         :param top_k: The maximum number of answers to return
-        :return: Dict containing question and answers
+        :return: Dict containing query and answers
         """
 
         # convert input to FARM format
         inputs = []
         for doc in documents:
             cur = QAInput(doc_text=doc.text,
-                          questions=Question(text=question,
+                          questions=Question(text=query,
                                              uid=doc.id))
             inputs.append(cur)
 
@@ -344,7 +366,7 @@ class FARMReader(BaseReader):
         )
         # assemble answers from all the different documents & format them.
         answers, max_no_ans_gap = self._extract_answers_of_predictions(predictions, top_k)
-        result = {"question": question,
+        result = {"query": query,
                   "no_ans_gap": max_no_ans_gap,
                   "answers": answers}
 
@@ -571,20 +593,21 @@ class FARMReader(BaseReader):
         Use loaded QA model to find answers for a question in the supplied list of Document.
         Returns dictionaries containing answers sorted by (desc.) probability.
         Example:
-
-            {
-                'question': 'Who is the father of Arya Stark?',
-                'answers':[
-                             {'answer': 'Eddard,',
-                             'context': " She travels with her father, Eddard, to King's Landing when he is ",
-                             'offset_answer_start': 147,
-                             'offset_answer_end': 154,
-                             'probability': 0.9787139466668613,
-                             'score': None,
-                             'document_id': '1337'
-                             },...
-                          ]
-            }
+         ```python
+            |{
+            |    'question': 'Who is the father of Arya Stark?',
+            |    'answers':[
+            |                 {'answer': 'Eddard,',
+            |                 'context': " She travels with her father, Eddard, to King's Landing when he is ",
+            |                 'offset_answer_start': 147,
+            |                 'offset_answer_end': 154,
+            |                 'probability': 0.9787139466668613,
+            |                 'score': None,
+            |                 'document_id': '1337'
+            |                 },...
+            |              ]
+            |}
+         ```
 
         :param question: Question string
         :param documents: List of documents as string type
