@@ -6,6 +6,7 @@ from pathlib import Path
 from tqdm import tqdm
 
 from haystack.document_store.base import BaseDocumentStore
+from haystack.document_store.elasticsearch import ElasticsearchDocumentStore
 from haystack import Document
 from haystack.retriever.base import BaseRetriever
 
@@ -52,14 +53,14 @@ class DensePassageRetriever(BaseRetriever):
         **Example:**
 
                 ```python
-                # remote model from FAIR
-                DensePassageRetriever(document_store=your_doc_store,	
-                >                    query_embedding_model="facebook/dpr-question_encoder-single-nq-base",	
-                >                    passage_embedding_model="facebook/dpr-ctx_encoder-single-nq-base")	
-                # or from local path
-                DensePassageRetriever(document_store=your_doc_store,	
-                >                    query_embedding_model="model_directory/question-encoder",	
-                >                   passage_embedding_model="model_directory/context-encoder")
+                |    # remote model from FAIR
+                |    DensePassageRetriever(document_store=your_doc_store,
+                |                          query_embedding_model="facebook/dpr-question_encoder-single-nq-base",
+                |                          passage_embedding_model="facebook/dpr-ctx_encoder-single-nq-base")
+                |    # or from local path
+                |    DensePassageRetriever(document_store=your_doc_store,
+                |                          query_embedding_model="model_directory/question-encoder",
+                |                          passage_embedding_model="model_directory/context-encoder")
                 ```
 
         :param document_store: An instance of DocumentStore from which to retrieve documents.
@@ -86,6 +87,11 @@ class DensePassageRetriever(BaseRetriever):
         self.max_seq_len_passage = max_seq_len_passage
         self.max_seq_len_query = max_seq_len_query
 
+        if document_store.similarity != "dot_product":
+            logger.warning(f"You are using a Dense Passage Retriever model with the {document_store.similarity} function. "
+                           "We recommend you use dot_product instead. "
+                           "This can be set when initializing the DocumentStore")
+
         if use_gpu and torch.cuda.is_available():
             self.device = torch.device("cuda")
         else:
@@ -112,7 +118,7 @@ class DensePassageRetriever(BaseRetriever):
                                                  metric="text_similarity_metric",
                                                  embed_title=self.embed_title,
                                                  num_hard_negatives=0,
-                                                 num_negatives=0)
+                                                 num_positives=1)
 
         prediction_head = TextSimilarityHead(similarity_function=similarity_function)
         self.model = BiAdaptiveModel(
@@ -127,13 +133,22 @@ class DensePassageRetriever(BaseRetriever):
         self.model.connect_heads_with_processor(self.processor.tasks, require_labels=False)
 
     def retrieve(self, query: str, filters: dict = None, top_k: int = 10, index: str = None) -> List[Document]:
+        """
+        Scan through documents in DocumentStore and return a small number documents
+        that are most relevant to the query.
+
+        :param query: The query
+        :param filters: A dictionary where the keys specify a metadata field and the value is a list of accepted values for that field
+        :param top_k: How many documents to return per query.
+        :param index: The name of the index in the DocumentStore from which to retrieve documents
+        """
         if index is None:
             index = self.document_store.index
         query_emb = self.embed_queries(texts=[query])
         documents = self.document_store.query_by_embedding(query_emb=query_emb[0], top_k=top_k, filters=filters, index=index)
         return documents
 
-    def _get_predictions(self, dicts, tokenizer):
+    def _get_predictions(self, dicts):
         """
         Feed a preprocessed dataset to the model and get the actual predictions (forward pass + formatting).
 
@@ -150,6 +165,8 @@ class DensePassageRetriever(BaseRetriever):
                     "external_id": '19930582'}, ...]
         :return: dictionary of embeddings for "passages" and "query"
         """
+
+
         dataset, tensor_names, baskets = self.processor.dataset_from_dicts(
             dicts, indices=[i for i in range(len(dicts))], return_baskets=True
         )
@@ -157,22 +174,23 @@ class DensePassageRetriever(BaseRetriever):
         data_loader = NamedDataLoader(
             dataset=dataset, sampler=SequentialSampler(dataset), batch_size=self.batch_size, tensor_names=tensor_names
         )
-        all_embeddings = {"query": torch.tensor([]).to(self.device), "passages": torch.tensor([]).to(self.device)}
+        all_embeddings = {"query": [], "passages": []}
         self.model.eval()
-        for i, batch in enumerate(tqdm(data_loader, desc=f"Inferencing Samples", unit=" Batches", disable=False)):
+        for i, batch in enumerate(tqdm(data_loader, desc=f"Creating Embeddings", unit=" Batches", disable=False)):
             batch = {key: batch[key].to(self.device) for key in batch}
 
             # get logits
             with torch.no_grad():
                 query_embeddings, passage_embeddings = self.model.forward(**batch)[0]
-                all_embeddings["query"] = torch.cat((all_embeddings["query"], query_embeddings), dim=0) \
-                                                     if isinstance(query_embeddings, torch.Tensor) else None
-                all_embeddings["passages"] = torch.cat((all_embeddings["passages"], passage_embeddings), dim=0) \
-                                                    if isinstance(passage_embeddings, torch.Tensor) else None
+                if query_embeddings is not None:
+                    all_embeddings["query"].append(query_embeddings.cpu().numpy())
+                if passage_embeddings is not None:
+                    all_embeddings["passages"].append(passage_embeddings.cpu().numpy())
 
-        # convert embeddings to numpy array
-        for k, v in all_embeddings.items():
-            all_embeddings[k] = v.cpu().numpy() if v!=None else None
+        if all_embeddings["passages"]:
+            all_embeddings["passages"] = np.concatenate(all_embeddings["passages"])
+        if all_embeddings["query"]:
+            all_embeddings["query"] = np.concatenate(all_embeddings["query"])
         return all_embeddings
 
     def embed_queries(self, texts: List[str]) -> List[np.array]:
@@ -183,7 +201,7 @@ class DensePassageRetriever(BaseRetriever):
         :return: Embeddings, one per input queries
         """
         queries = [{'query': q} for q in texts]
-        result = self._get_predictions(queries, self.query_tokenizer)["query"]
+        result = self._get_predictions(queries)["query"]
         return result
 
     def embed_passages(self, docs: List[Document]) -> List[np.array]:
@@ -199,7 +217,7 @@ class DensePassageRetriever(BaseRetriever):
             "label": d.meta["label"] if d.meta and "label" in d.meta else "positive",
             "external_id": d.id}]
         } for d in docs]
-        embeddings = self._get_predictions(passages, self.passage_tokenizer)["passages"]
+        embeddings = self._get_predictions(passages)["passages"]
 
         return embeddings
 
@@ -211,7 +229,7 @@ class DensePassageRetriever(BaseRetriever):
               batch_size: int = 2,
               embed_title: bool = True,
               num_hard_negatives: int = 1,
-              num_negatives: int = 0,
+              num_positives: int = 1,
               n_epochs: int = 3,
               evaluate_every: int = 1000,
               n_gpu: int = 1,
@@ -222,9 +240,9 @@ class DensePassageRetriever(BaseRetriever):
               grad_acc_steps: int = 1,
               optimizer_name: str = "TransformersAdamW",
               optimizer_correct_bias: bool = True,
-              save_dir: str = "../saved_models/dpr-tutorial",
-              query_encoder_save_dir: str = "lm1",
-              passage_encoder_save_dir: str = "lm2"
+              save_dir: str = "../saved_models/dpr",
+              query_encoder_save_dir: str = "query_encoder",
+              passage_encoder_save_dir: str = "passage_encoder"
               ):
         """
         train a DensePassageRetrieval model
@@ -235,7 +253,7 @@ class DensePassageRetriever(BaseRetriever):
         :param batch_size: total number of samples in 1 batch of data
         :param embed_title: whether to concatenate passage title with each passage. The default setting in official DPR embeds passage title with the corresponding passage
         :param num_hard_negatives: number of hard negative passages(passages which are very similar(high score by BM25) to query but do not contain the answer
-        :param num_negatives: number of negative passages(any random passage from dataset which do not contain answer to query)
+        :param num_positives: number of positive passages
         :param n_epochs: number of epochs to train the model on
         :param evaluate_every: number of training steps after evaluation is run
         :param n_gpu: number of gpus to train on
@@ -264,7 +282,7 @@ class DensePassageRetriever(BaseRetriever):
                                                  test_filename=test_filename,
                                                  embed_title=self.embed_title,
                                                  num_hard_negatives=num_hard_negatives,
-                                                 num_negatives=num_negatives)
+                                                 num_positives=num_positives)
 
         self.model.connect_heads_with_processor(self.processor.tasks, require_labels=True)
 
@@ -299,14 +317,24 @@ class DensePassageRetriever(BaseRetriever):
         trainer.train()
 
         self.model.save(Path(save_dir), lm1_name=query_encoder_save_dir, lm2_name=passage_encoder_save_dir)
-        self.processor.save(Path(save_dir))
+        self.query_tokenizer.save_pretrained(f"{save_dir}/{query_encoder_save_dir}")
+        self.passage_tokenizer.save_pretrained(f"{save_dir}/{passage_encoder_save_dir}")
 
-    def save(self, save_dir: Union[Path, str]):
+    def save(self, save_dir: Union[Path, str], query_encoder_dir: str = "query_encoder",
+             passage_encoder_dir: str = "passage_encoder"):
+        """
+        Save DensePassageRetriever to the specified directory.
+
+        :param save_dir: Directory to save to.
+        :param query_encoder_dir: Directory in save_dir that contains query encoder model.
+        :param passage_encoder_dir: Directory in save_dir that contains passage encoder model.
+        :return: None
+        """
         save_dir = Path(save_dir)
-        self.model.save(save_dir, lm1_name="query_encoder", lm2_name="passage_encoder")
+        self.model.save(save_dir, lm1_name=query_encoder_dir, lm2_name=passage_encoder_dir)
         save_dir = str(save_dir)
-        self.query_tokenizer.save_pretrained(save_dir + "/query_encoder")
-        self.passage_tokenizer.save_pretrained(save_dir + "/passage_encoder")
+        self.query_tokenizer.save_pretrained(save_dir + f"/{query_encoder_dir}")
+        self.passage_tokenizer.save_pretrained(save_dir + f"/{passage_encoder_dir}")
 
     @classmethod
     def load(cls,
@@ -319,13 +347,18 @@ class DensePassageRetriever(BaseRetriever):
              embed_title: bool = True,
              use_fast_tokenizers: bool = True,
              similarity_function: str = "dot_product",
+             query_encoder_dir: str = "query_encoder",
+             passage_encoder_dir: str = "passage_encoder"
              ):
+        """
+        Load DensePassageRetriever from the specified directory.
+        """
 
         load_dir = Path(load_dir)
         dpr = cls(
             document_store=document_store,
-            query_embedding_model=Path(load_dir) / "query_encoder",
-            passage_embedding_model=Path(load_dir) / "passage_encoder",
+            query_embedding_model=Path(load_dir) / query_encoder_dir,
+            passage_embedding_model=Path(load_dir) / passage_encoder_dir,
             max_seq_len_query=max_seq_len_query,
             max_seq_len_passage=max_seq_len_passage,
             use_gpu=use_gpu,
@@ -369,7 +402,6 @@ class EmbeddingRetriever(BaseRetriever):
         """
         self.document_store = document_store
         self.model_format = model_format
-        self.embedding_model = embedding_model
         self.pooling_strategy = pooling_strategy
         self.emb_extraction_layer = emb_extraction_layer
 
@@ -379,6 +411,18 @@ class EmbeddingRetriever(BaseRetriever):
                 embedding_model, task_type="embeddings", extraction_strategy=self.pooling_strategy,
                 extraction_layer=self.emb_extraction_layer, gpu=use_gpu, batch_size=4, max_seq_len=512, num_processes=0
             )
+            # Check that document_store has the right similarity function
+            similarity = document_store.similarity
+            # If we are using a sentence transformer model
+            if "sentence" in embedding_model.lower() and similarity != "cosine":
+                logger.warning(f"You seem to be using a Sentence Transformer with the {similarity} function. "
+                               f"We recommend using cosine instead. "
+                               f"This can be set when initializing the DocumentStore")
+            elif "dpr" in embedding_model.lower() and similarity != "dot_product":
+                logger.warning(f"You seem to be using a DPR model with the {similarity} function. "
+                               f"We recommend using dot_product instead. "
+                               f"This can be set when initializing the DocumentStore")
+
 
         elif model_format == "sentence_transformers":
             try:
@@ -394,10 +438,24 @@ class EmbeddingRetriever(BaseRetriever):
             else:
                 device = "cpu"
             self.embedding_model = SentenceTransformer(embedding_model, device=device)
+            if document_store.similarity != "cosine":
+                logger.warning(
+                    f"You are using a Sentence Transformer with the {document_store.similarity} function. "
+                    f"We recommend using cosine instead. "
+                    f"This can be set when initializing the DocumentStore")
         else:
             raise NotImplementedError
 
     def retrieve(self, query: str, filters: dict = None, top_k: int = 10, index: str = None) -> List[Document]:
+        """
+        Scan through documents in DocumentStore and return a small number documents
+        that are most relevant to the query.
+
+        :param query: The query
+        :param filters: A dictionary where the keys specify a metadata field and the value is a list of accepted values for that field
+        :param top_k: How many documents to return per query.
+        :param index: The name of the index in the DocumentStore from which to retrieve documents
+        """
         if index is None:
             index = self.document_store.index
         query_emb = self.embed(texts=[query])
@@ -414,19 +472,19 @@ class EmbeddingRetriever(BaseRetriever):
         """
 
         # for backward compatibility: cast pure str input
-        if type(texts) == str:
-            texts = [texts]  # type: ignore
-        assert type(texts) == list, "Expecting a list of texts, i.e. create_embeddings(texts=['text1',...])"
+        if isinstance(texts, str):
+            texts = [texts]
+        assert isinstance(texts, list), "Expecting a list of texts, i.e. create_embeddings(texts=['text1',...])"
 
         if self.model_format == "farm" or self.model_format == "transformers":
             # TODO: FARM's `sample_to_features_text` need to fix following warning -
             # tokenization_utils.py:460: FutureWarning: `is_pretokenized` is deprecated and will be removed in a future version, use `is_split_into_words` instead.
-            emb = self.embedding_model.inference_from_dicts(dicts=[{"text": t} for t in texts])  # type: ignore
+            emb = self.embedding_model.inference_from_dicts(dicts=[{"text": t} for t in texts])
             emb = [(r["vec"]) for r in emb]
         elif self.model_format == "sentence_transformers":
             # text is single string, sentence-transformers needs a list of strings
             # get back list of numpy embedding vectors
-            emb = self.embedding_model.encode(texts)  # type: ignore
+            emb = self.embedding_model.encode(texts)
             emb = [r for r in emb]
         return emb
 
