@@ -18,7 +18,7 @@ class ElasticsearchRetriever(BaseRetriever):
     def __init__(self, document_store: ElasticsearchDocumentStore, custom_query: str = None):
         """
         :param document_store: an instance of a DocumentStore to retrieve documents from.
-        :param custom_query: query string as per Elasticsearch DSL with a mandatory question placeholder($question).
+        :param custom_query: query string as per Elasticsearch DSL with a mandatory query placeholder(query).
 
                              Optionally, ES `filter` clause can be added where the values of `terms` are placeholders
                              that get substituted during runtime. The placeholder(${filter_name_1}, ${filter_name_2}..)
@@ -27,34 +27,43 @@ class ElasticsearchRetriever(BaseRetriever):
 
                                  **An example custom_query:**
                                  ```python
-                                 {
-                                >    "size": 10,
-                                >    "query": {
-                                >        "bool": {
-                                >            "should": [{"multi_match": {
-                                >                "query": "${question}",                 // mandatory $question placeholder
-                                >                "type": "most_fields",
-                                >                "fields": ["text", "title"]}}],
-                                >            "filter": [                                 // optional custom filters
-                                >                {"terms": {"year": "${years}"}},
-                                >                {"terms": {"quarter": "${quarters}"}},
-                                >                {"range": {"date": {"gte": "${date}"}}}
-                                >                ],
-                                >        }
-                                >    },
-                                }
+                                |    {
+                                |        "size": 10,
+                                |        "query": {
+                                |            "bool": {
+                                |                "should": [{"multi_match": {
+                                |                    "query": "${query}",                 // mandatory query placeholder
+                                |                    "type": "most_fields",
+                                |                    "fields": ["text", "title"]}}],
+                                |                "filter": [                                 // optional custom filters
+                                |                    {"terms": {"year": "${years}"}},
+                                |                    {"terms": {"quarter": "${quarters}"}},
+                                |                    {"range": {"date": {"gte": "${date}"}}}
+                                |                    ],
+                                |            }
+                                |        },
+                                |    }
                                  ```
 
                              **For this custom_query, a sample retrieve() could be:**
                              ```python
-                            self.retrieve(query="Why did the revenue increase?",
-                                >          filters={"years": ["2019"], "quarters": ["Q1", "Q2"]})
+                            |    self.retrieve(query="Why did the revenue increase?",
+                            |                  filters={"years": ["2019"], "quarters": ["Q1", "Q2"]})
                             ```
         """
         self.document_store: ElasticsearchDocumentStore = document_store
         self.custom_query = custom_query
 
     def retrieve(self, query: str, filters: dict = None, top_k: int = 10, index: str = None) -> List[Document]:
+        """
+        Scan through documents in DocumentStore and return a small number documents
+        that are most relevant to the query.
+
+        :param query: The query
+        :param filters: A dictionary where the keys specify a metadata field and the value is a list of accepted values for that field
+        :param top_k: How many documents to return per query.
+        :param index: The name of the index in the DocumentStore from which to retrieve documents
+        """
         if index is None:
             index = self.document_store.index
 
@@ -69,6 +78,15 @@ class ElasticsearchFilterOnlyRetriever(ElasticsearchRetriever):
     """
 
     def retrieve(self, query: str, filters: dict = None, top_k: int = 10, index: str = None) -> List[Document]:
+        """
+        Scan through documents in DocumentStore and return a small number documents
+        that are most relevant to the query.
+
+        :param query: The query
+        :param filters: A dictionary where the keys specify a metadata field and the value is a list of accepted values for that field
+        :param top_k: How many documents to return per query.
+        :param index: The name of the index in the DocumentStore from which to retrieve documents
+        """
         if index is None:
             index = self.document_store.index
         documents = self.document_store.query(query=None, filters=filters, top_k=top_k,
@@ -132,6 +150,18 @@ class TfidfRetriever(BaseRetriever):
         return indices_and_scores
 
     def retrieve(self, query: str, filters: dict = None, top_k: int = 10, index: str = None) -> List[Document]:
+        """
+        Scan through documents in DocumentStore and return a small number documents
+        that are most relevant to the query.
+
+        :param query: The query
+        :param filters: A dictionary where the keys specify a metadata field and the value is a list of accepted values for that field
+        :param top_k: How many documents to return per query.
+        :param index: The name of the index in the DocumentStore from which to retrieve documents
+        """
+        if self.df is None:
+            raise Exception("fit() needs to called before retrieve()")
+
         if filters:
             raise NotImplementedError("Filters are not implemented in TfidfRetriever.")
         if index:
@@ -141,7 +171,7 @@ class TfidfRetriever(BaseRetriever):
         indices_and_scores = self._calc_scores(query)
 
         # rank paragraphs
-        df_sliced = self.df.loc[indices_and_scores.keys()]  # type: ignore
+        df_sliced = self.df.loc[indices_and_scores.keys()]
         df_sliced = df_sliced[:top_k]
 
         logger.debug(
@@ -165,6 +195,9 @@ class TfidfRetriever(BaseRetriever):
         return documents
 
     def fit(self):
+        """
+        Performing training on this class according to the TF-IDF algorithm.
+        """
         if not self.paragraphs or len(self.paragraphs) == 0:
             self.paragraphs = self._get_all_paragraphs()
             if not self.paragraphs or len(self.paragraphs) == 0:
